@@ -32,6 +32,7 @@
 #include "drumkv1_reverb.h"
 
 #include "drumkv1_config.h"
+#include "drumkv1_controls.h"
 #include "drumkv1_programs.h"
 
 
@@ -637,7 +638,7 @@ class drumkv1_elem : public drumkv1_list<drumkv1_elem>
 {
 public:
 
-	drumkv1_elem(uint32_t iSampleRate, int key);
+	drumkv1_elem(drumkv1 *pDrumk, float srate, int key);
 
 	drumkv1_element element;
 
@@ -657,14 +658,14 @@ public:
 
 	float params[3][drumkv1::NUM_ELEMENT_PARAMS];
 
-	void updateEnvTimes(uint32_t iSampleRate);
+	void updateEnvTimes(float srate);
 };
 
 
 // synth element
 
-drumkv1_elem::drumkv1_elem ( uint32_t iSampleRate, int key )
-	: element(this)
+drumkv1_elem::drumkv1_elem ( drumkv1 *pDrumk, float srate, int key )
+	: element(this), gen1_sample(pDrumk)
 {
 	// element parameter value set
 	for (uint32_t i = 0; i < drumkv1::NUM_ELEMENT_PARAMS; ++i)
@@ -682,17 +683,17 @@ drumkv1_elem::drumkv1_elem ( uint32_t iSampleRate, int key )
 	}
 
 	// element sample rate
-	gen1_sample.setSampleRate(iSampleRate);
-	lfo1_wave.setSampleRate(iSampleRate);
+	gen1_sample.setSampleRate(srate);
+	lfo1_wave.setSampleRate(srate);
 
-	updateEnvTimes(iSampleRate);
+	updateEnvTimes(srate);
 }
 
 
-void drumkv1_elem::updateEnvTimes ( uint32_t iSampleRate )
+void drumkv1_elem::updateEnvTimes ( float srate )
 {
 	// element envelope range times in frames
-	const float srate_ms = 0.001f * float(iSampleRate);
+	const float srate_ms = 0.001f * srate;
 
 	float envtime_msecs = 10000.0f * gen1.envtime0;
 	if (envtime_msecs < MIN_ENV_MSECS)
@@ -760,15 +761,15 @@ class drumkv1_impl
 {
 public:
 
-	drumkv1_impl(drumkv1 *pDrumk, uint16_t iChannels, uint32_t iSampleRate);
+	drumkv1_impl(drumkv1 *pDrumk, uint16_t nchannels, float srate);
 
 	~drumkv1_impl();
 
-	void setChannels(uint16_t iChannels);
+	void setChannels(uint16_t nchannels);
 	uint16_t channels() const;
 
-	void setSampleRate(uint32_t iSampleRate);
-	uint32_t sampleRate() const;
+	void setSampleRate(float srate);
+	float sampleRate() const;
 
 	drumkv1_element *addElement(int key);
 	drumkv1_element *element(int key) const;
@@ -793,6 +794,7 @@ public:
 	void setParamValue(drumkv1::ParamIndex index, float fValue);
 	float paramValue(drumkv1::ParamIndex index) const;
 
+	drumkv1_controls *controls();
 	drumkv1_programs *programs();
 
 	void process_midi(uint8_t *data, uint32_t size);
@@ -833,11 +835,14 @@ protected:
 
 private:
 
+	drumkv1 *m_pDrumk;
+
 	drumkv1_config   m_config;
+	drumkv1_controls m_controls;
 	drumkv1_programs m_programs;
 
-	uint16_t m_iChannels;
-	uint32_t m_iSampleRate;
+	uint16_t m_nchannels;
+	float    m_srate;
 
 	drumkv1_ctl m_ctl;
 
@@ -877,8 +882,8 @@ private:
 // synth engine constructor
 
 drumkv1_impl::drumkv1_impl (
-	drumkv1 *pDrumk, uint16_t iChannels, uint32_t iSampleRate )
-	: m_programs(pDrumk)
+	drumkv1 *pDrumk, uint16_t nchannels, float srate )
+	: m_pDrumk(pDrumk), m_controls(pDrumk), m_programs(pDrumk)
 {
 	// allocate voice pool.
 	m_voices = new drumkv1_voice * [MAX_VOICES];
@@ -910,14 +915,15 @@ drumkv1_impl::drumkv1_impl (
 	m_del.bpmsync0 = 0.0f;
 	m_del.bpm0 = 0;
 
-	// load programs database...
+	// load controllers & programs database...
+	m_config.loadControls(&m_controls);
 	m_config.loadPrograms(&m_programs);
 
 	// number of channels
-	setChannels(iChannels);
+	setChannels(nchannels);
 
 	// set default sample rate
-	setSampleRate(iSampleRate);
+	setSampleRate(srate);
 
 	// init default element (eg. 36=Bass Drum 1)
 	clearElements();
@@ -963,9 +969,9 @@ drumkv1_impl::~drumkv1_impl (void)
 }
 
 
-void drumkv1_impl::setChannels ( uint16_t iChannels )
+void drumkv1_impl::setChannels ( uint16_t nchannels )
 {
-	m_iChannels = iChannels;
+	m_nchannels = nchannels;
 
 	// deallocate flangers
 	if (m_flanger) {
@@ -995,20 +1001,20 @@ void drumkv1_impl::setChannels ( uint16_t iChannels )
 
 uint16_t drumkv1_impl::channels (void) const
 {
-	return m_iChannels;
+	return m_nchannels;
 }
 
 
-void drumkv1_impl::setSampleRate ( uint32_t iSampleRate )
+void drumkv1_impl::setSampleRate ( float srate )
 {
 	// set internal sample rate
-	m_iSampleRate = iSampleRate;
+	m_srate = srate;
 }
 
 
-uint32_t drumkv1_impl::sampleRate (void) const
+float drumkv1_impl::sampleRate (void) const
 {
-	return m_iSampleRate;
+	return m_srate;
 }
 
 
@@ -1018,7 +1024,7 @@ drumkv1_element *drumkv1_impl::addElement ( int key )
 	if (key >= 0 && key < MAX_NOTES) {
 		elem = m_elems[key];
 		if (elem == 0) {
-			elem = new drumkv1_elem(m_iSampleRate, key);
+			elem = new drumkv1_elem(m_pDrumk, m_srate, key);
 			m_elem_list.append(elem);
 			m_elems[key] = elem;
 		}
@@ -1061,12 +1067,11 @@ void drumkv1_impl::setCurrentElement ( int key )
 		if (elem) {
 			drumkv1_element *element = &(elem->element);
 			for (uint32_t i = 0; i < drumkv1::NUM_ELEMENT_PARAMS; ++i) {
-				drumkv1::ParamIndex index = drumkv1::ParamIndex(i);
+				const drumkv1::ParamIndex index = drumkv1::ParamIndex(i);
 				if (index == drumkv1::GEN1_SAMPLE)
 					continue;
-				float *pfParam = element->paramPort(index);
+				float *pfParam = m_params[i];
 				if (pfParam) {
-					m_params[i] = pfParam;
 					elem->params[1][i] = *pfParam;
 					element->setParamPort(index, &(elem->params[1][i]));
 				}
@@ -1078,12 +1083,12 @@ void drumkv1_impl::setCurrentElement ( int key )
 		if (elem) {
 			drumkv1_element *element = &(elem->element);
 			for (uint32_t i = 0; i < drumkv1::NUM_ELEMENT_PARAMS; ++i) {
-				drumkv1::ParamIndex index = drumkv1::ParamIndex(i);
+				const drumkv1::ParamIndex index = drumkv1::ParamIndex(i);
 				if (index == drumkv1::GEN1_SAMPLE)
 					continue;
 				float *pfParam = m_params[i];
 				if (pfParam) {
-					*pfParam = elem->params[1][i];
+					*pfParam = elem->params[1][i]; // LV2:BUG?
 					element->setParamPort(index, pfParam);
 				}
 			}
@@ -1121,7 +1126,7 @@ void drumkv1_impl::clearElements (void)
 
 #if 0
 	// init default element (eg. 36=Bass Drum 1)
-	m_elem_list.append(new drumkv1_elem(m_iSampleRate, 36));
+	m_elem_list.append(new drumkv1_elem(m_srate, 36));
 	m_elem = m_elem_list.next();
 	m_elems[36] = m_elem;
 #endif
@@ -1202,10 +1207,31 @@ void drumkv1_impl::setParamPort ( drumkv1::ParamIndex index, float *pfParam )
 	case drumkv1::DYN1_COMPRESS:  m_dyn.compress  = pfParam; break;
 	case drumkv1::DYN1_LIMITER:   m_dyn.limiter   = pfParam; break;
 	default:
-		if (m_elem)
+		if (m_elem) {
 			m_elem->element.setParamPort(index, pfParam);
-		else
-			m_params[index] = pfParam;
+			// check null connections.
+			if (pfParam != &s_fDummy)
+			switch (index) {
+			case drumkv1::DCA1_VOLUME:
+			case drumkv1::OUT1_VOLUME:
+				m_elem->vol1.reset(m_elem->out1.volume,
+					m_elem->dca1.volume,
+					&m_ctl.volume,
+					&m_elem->aux1.volume);
+				break;
+			case drumkv1::OUT1_WIDTH:
+				m_elem->wid1.reset(m_elem->out1.width);
+				break;
+			case drumkv1::OUT1_PANNING:
+				m_elem->pan1.reset(m_elem->out1.panning,
+					&m_ctl.panning,
+					&m_elem->aux1.panning);
+				break;
+			default:
+				break;
+			}
+		}
+		m_params[index] = pfParam;
 		break;
 	}
 }
@@ -1250,10 +1276,7 @@ float *drumkv1_impl::paramPort ( drumkv1::ParamIndex index ) const
 	case drumkv1::DYN1_COMPRESS:  pfParam = m_dyn.compress;  break;
 	case drumkv1::DYN1_LIMITER:   pfParam = m_dyn.limiter;   break;
 	default:
-		if (m_elem)
-			pfParam = m_elem->element.paramPort(index);
-		else
-			pfParam = m_params[index];
+		pfParam = m_params[index];
 		break;
 	}
 
@@ -1265,7 +1288,7 @@ void drumkv1_impl::setParamValue ( drumkv1::ParamIndex index, float fValue )
 {
 	float *pfParamPort = paramPort(index);
 	if (pfParamPort)
-		*pfParamPort = fValue;
+		*pfParamPort = fValue; // LV2:BUG?
 }
 
 
@@ -1280,163 +1303,180 @@ float drumkv1_impl::paramValue ( drumkv1::ParamIndex index ) const
 
 void drumkv1_impl::process_midi ( uint8_t *data, uint32_t size )
 {
-	// check data size (#1)
-	if (size < 2)
-		return;
+	for (uint32_t i = 0; i < size; ++i) {
 
-	// channel filter
-	const int channel = (data[0] & 0x0f) + 1;
+		// channel status
+		const int channel = (data[i] & 0x0f) + 1;
+		const int status  = (data[i] & 0xf0);
 
-	const int ch = int(*m_def.channel);
-	const int on = (ch == 0 || ch == channel);
+		// channel filter
+		const int ch = int(*m_def.channel);
+		const int on = (ch == 0 || ch == channel);
 
-	if (!on)
-		return;
+		// all system common/real-time ignored
+		if (status == 0xf0)
+			continue;
 
-	// note on
-	const int status = (data[0] & 0xf0);
-	const int key    = (data[1] & 0x7f);
+		// check data size (#1)
+		if (++i >= size)
+			break;
+
+		const int key = (data[i] & 0x7f);
+
+		// program change
+		if (status == 0xc0) {
+			if (on) m_programs.prog_change(key);
+			continue;
+		}
 
-	// program change
-	if (status == 0xc0)
-		m_programs.prog_change(key);
-	else
-	if (status == 0xd0) {
 		// channel aftertouch
-		m_ctl.pressure = float(key) / 127.0f;
-	}
-
-	// check data size (#2)
-	if (size < 3)
-		return;
-
-	const int value  = (data[2] & 0x7f);
-
-	// note on
-	if (status == 0x90 && value > 0) {
-		drumkv1_voice *pv = m_notes[key];
-		if (pv && !(int(*m_def.noteoff) > 0)) {
-			drumkv1_elem *elem = pv->elem;
-			// retrigger fast release
-			elem->dcf1.env.note_off_fast(&pv->dcf1_env);
-			elem->lfo1.env.note_off_fast(&pv->lfo1_env);
-			elem->dca1.env.note_off_fast(&pv->dca1_env);
-			m_notes[key] = 0;
-			pv->note = -1;
+		if (status == 0xd0) {
+			if (on) m_ctl.pressure = float(key) / 127.0f;
+			continue;
 		}
-		// find free voice
-		pv = alloc_voice(key);
-		if (pv) {
-			drumkv1_elem *elem = pv->elem;
-			// waveform
-			pv->note = key;
-			// velocity
-			const float vel = float(value) / 127.0f;
-			// quadratic velocity law
-			pv->vel  = drumkv1_velocity(vel * vel, *m_def.velocity);
-			// pressure/aftertouch
-			pv->pre = 0.0f;
-			pv->dca1_pre.reset(m_def.pressure, &m_ctl.pressure, &pv->pre);
-			// generate
-			pv->gen1.start();
-			// frequencies
-			const float freq1 = float(key)
-				+ *elem->gen1.coarse * COARSE_SCALE
-				+ *elem->gen1.fine * FINE_SCALE;
-			pv->gen1_freq = drumkv1_freq(freq1);
-			// filters
-			const int type1 = int(*elem->dcf1.type);
-			pv->dcf11.reset(drumkv1_filter1::Type(type1));
-			pv->dcf12.reset(drumkv1_filter1::Type(type1));
-			pv->dcf13.reset(drumkv1_filter2::Type(type1));
-			pv->dcf14.reset(drumkv1_filter2::Type(type1));
-			// envelopes
-			elem->dcf1.env.start(&pv->dcf1_env);
-			elem->lfo1.env.start(&pv->lfo1_env);
-			elem->dca1.env.start(&pv->dca1_env);
-			// lfos
-			pv->lfo1_sample = pv->lfo1.start();
-			// allocated
-			m_notes[key] = pv;
-			// group management
-			pv->group = int(*elem->gen1.group * 100.0f) - 1;
-			if (pv->group >= 0) {
-				drumkv1_voice *pv_group = m_group[pv->group];
-				if (pv_group && pv_group->note >= 0 && pv_group->note != key) {
-					drumkv1_elem *elem_group = pv_group->elem;
-					// retrigger fast release
-					elem_group->dcf1.env.note_off_fast(&pv_group->dcf1_env);
-					elem_group->lfo1.env.note_off_fast(&pv_group->lfo1_env);
-					elem_group->dca1.env.note_off_fast(&pv_group->dca1_env);
-					m_notes[pv_group->note] = 0;
-					pv_group->note = -1;
-				}
-				m_group[pv->group] = pv;
-			}
-		}
-	}
-	// note off
-	else if (status == 0x80 || (status == 0x90 && value == 0)) {
-		if (int(*m_def.noteoff) > 0) {
+
+		// check data size (#2)
+		if (++i >= size)
+			break;
+
+		// channel filter
+		if (!on)
+			continue;
+
+		const int value = (data[i] & 0x7f);
+
+		// note on
+		if (status == 0x90 && value > 0) {
 			drumkv1_voice *pv = m_notes[key];
-			if (pv && pv->note >= 0) {
-				if (pv->dca1_env.stage != drumkv1_env::Decay2) {
-					drumkv1_elem *elem = pv->elem;
-					elem->dca1.env.note_off(&pv->dca1_env);
-					elem->dcf1.env.note_off(&pv->dcf1_env);
-					elem->lfo1.env.note_off(&pv->lfo1_env);
+			if (pv && !(int(*m_def.noteoff) > 0)) {
+				drumkv1_elem *elem = pv->elem;
+				// retrigger fast release
+				elem->dcf1.env.note_off_fast(&pv->dcf1_env);
+				elem->lfo1.env.note_off_fast(&pv->lfo1_env);
+				elem->dca1.env.note_off_fast(&pv->dca1_env);
+				m_notes[key] = 0;
+				pv->note = -1;
+			}
+			// find free voice
+			pv = alloc_voice(key);
+			if (pv) {
+				drumkv1_elem *elem = pv->elem;
+				// waveform
+				pv->note = key;
+				// velocity
+				const float vel = float(value) / 127.0f;
+				// quadratic velocity law
+				pv->vel  = drumkv1_velocity(vel * vel, *m_def.velocity);
+				// pressure/aftertouch
+				pv->pre = 0.0f;
+				pv->dca1_pre.reset(m_def.pressure, &m_ctl.pressure, &pv->pre);
+				// generate
+				pv->gen1.start();
+				// frequencies
+				const float freq1 = float(key)
+					+ *elem->gen1.coarse * COARSE_SCALE
+					+ *elem->gen1.fine * FINE_SCALE;
+				pv->gen1_freq = drumkv1_freq(freq1);
+				// filters
+				const int type1 = int(*elem->dcf1.type);
+				pv->dcf11.reset(drumkv1_filter1::Type(type1));
+				pv->dcf12.reset(drumkv1_filter1::Type(type1));
+				pv->dcf13.reset(drumkv1_filter2::Type(type1));
+				pv->dcf14.reset(drumkv1_filter2::Type(type1));
+				// envelopes
+				elem->dcf1.env.start(&pv->dcf1_env);
+				elem->lfo1.env.start(&pv->lfo1_env);
+				elem->dca1.env.start(&pv->dca1_env);
+				// lfos
+				pv->lfo1_sample = pv->lfo1.start();
+				// allocated
+				m_notes[key] = pv;
+				// group management
+				pv->group = int(*elem->gen1.group * 100.0f) - 1;
+				if (pv->group >= 0) {
+					drumkv1_voice *pv_group = m_group[pv->group];
+					if (pv_group && pv_group->note >= 0 && pv_group->note != key) {
+						drumkv1_elem *elem_group = pv_group->elem;
+						// retrigger fast release
+						elem_group->dcf1.env.note_off_fast(&pv_group->dcf1_env);
+						elem_group->lfo1.env.note_off_fast(&pv_group->lfo1_env);
+						elem_group->dca1.env.note_off_fast(&pv_group->dca1_env);
+						m_notes[pv_group->note] = 0;
+						pv_group->note = -1;
+					}
+					m_group[pv->group] = pv;
 				}
 			}
 		}
-	}
-	// key pressure/poly.aftertouch
-	else if (status == 0xa0) {
-		drumkv1_voice *pv = m_notes[key];
-		if (pv && pv->note >= 0)
-			pv->pre = *m_def.pressure * float(value) / 127.0f;
-	}
-	// control change
-	else if (status == 0xb0) {
-	switch (key) {
-		case 0x00:
-			// bank-select MSB (cc#0)
-			m_programs.bank_select_msb(value);
-			break;
-		case 0x01:
-			// modulation wheel (cc#1)
-			m_ctl.modwheel = *m_def.modwheel * float(value) / 127.0f;
-			break;
-		case 0x07:
-			// channel volume (cc#7)
-			m_ctl.volume = float(value) / 127.0f;
-			break;
-		case 0x0a:
-			// channel panning (cc#10)
-			m_ctl.panning = float(value - 64) / 64.0f;
-			break;
-		case 0x20:
-			// bank-select LSB (cc#32)
-			m_programs.bank_select_lsb(value);
-			break;
-		case 0x78:
-			// all sound off (cc#120)
-			allSoundOff();
-			break;
-		case 0x79:
-			// all controllers off (cc#121)
-			allControllersOff();
-			break;
-		case 0x7b:
-			// all notes off (cc#123)
-			allNotesOff();
-			break;
+		// note off
+		else if (status == 0x80 || (status == 0x90 && value == 0)) {
+			if (int(*m_def.noteoff) > 0) {
+				drumkv1_voice *pv = m_notes[key];
+				if (pv && pv->note >= 0) {
+					if (pv->dca1_env.stage != drumkv1_env::Decay2) {
+						drumkv1_elem *elem = pv->elem;
+						elem->dca1.env.note_off(&pv->dca1_env);
+						elem->dcf1.env.note_off(&pv->dcf1_env);
+						elem->lfo1.env.note_off(&pv->lfo1_env);
+					}
+				}
+			}
 		}
+		// key pressure/poly.aftertouch
+		else if (status == 0xa0) {
+			drumkv1_voice *pv = m_notes[key];
+			if (pv && pv->note >= 0)
+				pv->pre = *m_def.pressure * float(value) / 127.0f;
+		}
+		// control change
+		else if (status == 0xb0) {
+		switch (key) {
+			case 0x00:
+				// bank-select MSB (cc#0)
+				m_programs.bank_select_msb(value);
+				break;
+			case 0x01:
+				// modulation wheel (cc#1)
+				m_ctl.modwheel = *m_def.modwheel * float(value) / 127.0f;
+				break;
+			case 0x07:
+				// channel volume (cc#7)
+				m_ctl.volume = float(value) / 127.0f;
+				break;
+			case 0x0a:
+				// channel panning (cc#10)
+				m_ctl.panning = float(value - 64) / 64.0f;
+				break;
+			case 0x20:
+				// bank-select LSB (cc#32)
+				m_programs.bank_select_lsb(value);
+				break;
+			case 0x78:
+				// all sound off (cc#120)
+				allSoundOff();
+				break;
+			case 0x79:
+				// all controllers off (cc#121)
+				allControllersOff();
+				break;
+			case 0x7b:
+				// all notes off (cc#123)
+				allNotesOff();
+				break;
+			}
+			// process controllers...
+			m_controls.process_enqueue(channel, key, value);
+		}
+		// pitch bend
+		else if (status == 0xe0) {
+			const float pitchbend = float(key + (value << 7) - 0x2000) / 8192.0f;
+			m_ctl.pitchbend = drumkv1_pow2f(*m_def.pitchbend * pitchbend);
+		}
+
 	}
-	// pitch bend
-	else if (status == 0xe0) {
-		const float pitchbend = float(key + (value << 7) - 0x2000) / 8192.0f;
-		m_ctl.pitchbend = drumkv1_pow2f(*m_def.pitchbend * pitchbend);
-	}
+
+	// process pending controllers...
+	m_controls.process_dequeue();
 }
 
 
@@ -1452,20 +1492,20 @@ void drumkv1_impl::allControllersOff (void)
 
 void drumkv1_impl::allSoundOff (void)
 {
-	m_chorus.setSampleRate(m_iSampleRate);
+	m_chorus.setSampleRate(m_srate);
 	m_chorus.reset();
 
-	for (uint16_t k = 0; k < m_iChannels; ++k) {
-		m_phaser[k].setSampleRate(m_iSampleRate);
-		m_delay[k].setSampleRate(m_iSampleRate);
-		m_comp[k].setSampleRate(m_iSampleRate);
+	for (uint16_t k = 0; k < m_nchannels; ++k) {
+		m_phaser[k].setSampleRate(m_srate);
+		m_delay[k].setSampleRate(m_srate);
+		m_comp[k].setSampleRate(m_srate);
 		m_flanger[k].reset();
 		m_phaser[k].reset();
 		m_delay[k].reset();
 		m_comp[k].reset();
 	}
 
-	m_reverb.setSampleRate(m_iSampleRate);
+	m_reverb.setSampleRate(m_srate);
 	m_reverb.reset();
 }
 
@@ -1520,7 +1560,7 @@ void drumkv1_impl::resetParamValues ( bool bSwap )
 
 void drumkv1_impl::reset (void)
 {
-#if 1//--legacy support < 0.3.0.4
+#if 0//--legacy support < 0.3.0.4
 	if (*m_del.bpm < 3.6f)
 		*m_del.bpm *= 100.0f;
 #endif
@@ -1539,29 +1579,40 @@ void drumkv1_impl::reset (void)
 
 	// flangers
 	if (m_flanger == 0)
-		m_flanger = new drumkv1_fx_flanger [m_iChannels];
+		m_flanger = new drumkv1_fx_flanger [m_nchannels];
 
 	// phasers
 	if (m_phaser == 0)
-		m_phaser = new drumkv1_fx_phaser [m_iChannels];
+		m_phaser = new drumkv1_fx_phaser [m_nchannels];
 
 	// delays
 	if (m_delay == 0)
-		m_delay = new drumkv1_fx_delay [m_iChannels];
+		m_delay = new drumkv1_fx_delay [m_nchannels];
 
 	// compressors
 	if (m_comp == 0)
-		m_comp = new drumkv1_fx_comp [m_iChannels];
+		m_comp = new drumkv1_fx_comp [m_nchannels];
 
 	// reverbs
 	m_reverb.reset();
 
+	// controllers reset.
+	m_controls.reset();
+
 	allSoundOff();
 //	allControllersOff();
 	allNotesOff();
 }
 
 
+// controllers accessor
+
+drumkv1_controls *drumkv1_impl::controls (void)
+{
+	return &m_controls;
+}
+
+
 // programs accessor
 
 drumkv1_programs *drumkv1_impl::programs (void)
@@ -1574,13 +1625,13 @@ drumkv1_programs *drumkv1_impl::programs (void)
 
 void drumkv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 {
-	float *v_outs[m_iChannels];
+	float *v_outs[m_nchannels];
 
 	// buffer i/o transfer
 
 	uint16_t k;
 
-	for (k = 0; k < m_iChannels; ++k)
+	for (k = 0; k < m_nchannels; ++k)
 		::memcpy(outs[k], ins[k], nframes * sizeof(float));
 
 	drumkv1_elem *elem = m_elem_list.next();
@@ -1593,7 +1644,7 @@ void drumkv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 	#endif
 		if (elem->gen1.envtime0 != *elem->gen1.envtime) {
 			elem->gen1.envtime0  = *elem->gen1.envtime;
-			elem->updateEnvTimes(m_iSampleRate);
+			elem->updateEnvTimes(m_srate);
 		}
 		elem->gen1_sample.reverse_test(*elem->gen1.reverse > 0.5f);
 		elem->lfo1_wave.reset_test(
@@ -1627,7 +1678,7 @@ void drumkv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 
 		// output buffers
 
-		for (k = 0; k < m_iChannels; ++k)
+		for (k = 0; k < m_nchannels; ++k)
 			v_outs[k] = outs[k];
 
 		uint32_t nblock = nframes;
@@ -1698,7 +1749,7 @@ void drumkv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 				const float out2
 					= vol1 * (mid1 - sid1 * wid1) * elem->pan1.value(j, 1);
 
-				for (k = 0; k < m_iChannels; ++k)
+				for (k = 0; k < m_nchannels; ++k)
 					*v_outs[k]++ += (k & 1 ? out2 : out1);
 
 				if (j == 0) {
@@ -1745,13 +1796,13 @@ void drumkv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 	}
 
 	// chorus
-	if (m_iChannels > 1) {
+	if (m_nchannels > 1) {
 		m_chorus.process(outs[0], outs[1], nframes, *m_cho.wet,
 			*m_cho.delay, *m_cho.feedb, *m_cho.rate, *m_cho.mod);
 	}
 
 	// effects
-	for (k = 0; k < m_iChannels; ++k) {
+	for (k = 0; k < m_nchannels; ++k) {
 		float *in = outs[k];
 		// flanger
 		m_flanger[k].process(in, nframes, *m_fla.wet,
@@ -1765,13 +1816,13 @@ void drumkv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 	}
 
 	// reverb
-	if (m_iChannels > 1) {
+	if (m_nchannels > 1) {
 		m_reverb.process(outs[0], outs[1], nframes, *m_rev.wet,
 			*m_rev.feedb, *m_rev.room, *m_rev.damp, *m_rev.width);
 	}
 
 	// dynamics
-	for (k = 0; k < m_iChannels; ++k) {
+	for (k = 0; k < m_nchannels; ++k) {
 		float *in = outs[k];
 		// compressor
 		if (int(*m_dyn.compress) > 0)
@@ -1793,6 +1844,8 @@ void drumkv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 		elem->vol1.process(nframes);
 		elem = elem->next();
 	}
+
+	m_controls.process(nframes);
 }
 
 
@@ -1801,9 +1854,9 @@ void drumkv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 // drumkv1 - decl.
 //
 
-drumkv1::drumkv1 ( uint16_t iChannels, uint32_t iSampleRate )
+drumkv1::drumkv1 ( uint16_t nchannels, float srate )
 {
-	m_pImpl = new drumkv1_impl(this, iChannels, iSampleRate);
+	m_pImpl = new drumkv1_impl(this, nchannels, srate);
 }
 
 
@@ -1813,9 +1866,9 @@ drumkv1::~drumkv1 (void)
 }
 
 
-void drumkv1::setChannels ( uint16_t iChannels )
+void drumkv1::setChannels ( uint16_t nchannels )
 {
-	m_pImpl->setChannels(iChannels);
+	m_pImpl->setChannels(nchannels);
 }
 
 
@@ -1825,13 +1878,13 @@ uint16_t drumkv1::channels (void) const
 }
 
 
-void drumkv1::setSampleRate ( uint32_t iSampleRate )
+void drumkv1::setSampleRate ( float srate )
 {
-	m_pImpl->setSampleRate(iSampleRate);
+	m_pImpl->setSampleRate(srate);
 }
 
 
-uint32_t drumkv1::sampleRate (void) const
+float drumkv1::sampleRate (void) const
 {
 	return m_pImpl->sampleRate();
 }
@@ -1947,6 +2000,14 @@ void drumkv1::resetParamValues ( bool bSwap )
 }
 
 
+// controllers accessor
+
+drumkv1_controls *drumkv1::controls (void) const
+{
+	return m_pImpl->controls();
+}
+
+
 // programs accessor
 
 drumkv1_programs *drumkv1::programs (void) const
@@ -1971,7 +2032,7 @@ drumkv1_element::drumkv1_element ( drumkv1_elem *pElem )
 	: m_pElem(pElem)
 {
 	for (uint32_t i = 0; i < drumkv1::NUM_ELEMENT_PARAMS; ++i) {
-		drumkv1::ParamIndex index = drumkv1::ParamIndex(i);
+		const drumkv1::ParamIndex index = drumkv1::ParamIndex(i);
 		setParamPort(index, &(m_pElem->params[1][i]));
 	}
 }
@@ -2140,7 +2201,7 @@ float drumkv1_element::paramValue ( drumkv1::ParamIndex index, int pset )
 void drumkv1_element::resetParamValues ( bool bSwap )
 {
 	for (uint32_t i = 0; i < drumkv1::NUM_ELEMENT_PARAMS; ++i) {
-		drumkv1::ParamIndex index = drumkv1::ParamIndex(i);
+		const drumkv1::ParamIndex index = drumkv1::ParamIndex(i);
 		const float	fOldValue = m_pElem->params[1][index];
 		const float fNewValue = m_pElem->params[2][index];
 		m_pElem->params[2][index] = fOldValue;
