@@ -31,6 +31,7 @@ class drumkv1_impl;
 class drumkv1_elem;
 class drumkv1_element;
 class drumkv1_sample;
+class drumkv1_controls;
 class drumkv1_programs;
 
 
@@ -42,15 +43,15 @@ class drumkv1
 {
 public:
 
-	drumkv1(uint16_t iChannels = 2, uint32_t iSampleRate = 44100);
+	drumkv1(uint16_t nchannels = 2, float srate = 44100.0f);
 
 	~drumkv1();
 
-	void setChannels(uint16_t iChannels);
+	void setChannels(uint16_t nchannels);
 	uint16_t channels() const;
 
-	void setSampleRate(uint32_t iSampleRate);
-	uint32_t sampleRate() const;
+	void setSampleRate(float srate);
+	float sampleRate() const;
 
 	drumkv1_element *addElement(int key);
 	drumkv1_element *element(int key) const;
@@ -157,6 +158,7 @@ public:
 	void resetParamValues(bool bSwap);
 	void reset();
 
+	drumkv1_controls *controls() const;
 	drumkv1_programs *programs() const;
 
 	void process_midi(uint8_t *data, uint32_t size);
