@@ -23,6 +23,9 @@
 #include "drumkv1_config.h"
 #include "drumkv1_param.h"
 
+#include "drumkv1_programs.h"
+#include "drumkv1_controls.h"
+
 #include <jack/midiport.h>
 
 #include <stdio.h>
@@ -163,6 +166,9 @@ drumkv1_jack::drumkv1_jack (void) : drumkv1(2)
 
 	m_bpm = 0.0f;
 
+	drumkv1::programs()->enabled(true);
+	drumkv1::controls()->enabled(true);
+
 	open(DRUMKV1_TITLE);
 	activate();
 }
@@ -272,7 +278,7 @@ void drumkv1_jack::open ( const char *client_id )
 {
 	// init param ports
 	for (uint32_t i = 0; i < drumkv1::NUM_PARAMS; ++i) {
-		drumkv1::ParamIndex index = drumkv1::ParamIndex(i);
+		const drumkv1::ParamIndex index = drumkv1::ParamIndex(i);
 		m_params[i] = drumkv1_param::paramDefaultValue(index);
 		drumkv1::setParamPort(index, &m_params[i]);
 	}
@@ -283,11 +289,11 @@ void drumkv1_jack::open ( const char *client_id )
 		return;
 
 	// set sample rate
-	drumkv1::setSampleRate(jack_get_sample_rate(m_client));
+	drumkv1::setSampleRate(float(jack_get_sample_rate(m_client)));
 //	drumkv1::reset();
 
 	// register audio ports & buffers
-	uint16_t nchannels = drumkv1::channels();
+	const uint16_t nchannels = drumkv1::channels();
 
 	m_audio_ins  = new jack_port_t * [nchannels];
 	m_audio_outs = new jack_port_t * [nchannels];
