@@ -1,7 +1,7 @@
 // drumkv1_jack.h
 //
 /****************************************************************************
-   Copyright (C) 2012-2014, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2015, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -101,7 +101,7 @@ private:
 
 
 //-------------------------------------------------------------------------
-// drumkv1_application -- Singleton application instance.
+// drumkv1_jack_application -- Singleton application instance.
 //
 
 #include <QObject>
@@ -117,17 +117,17 @@ class drumkv1_nsm;
 #endif
 
 
-class drumkv1_application : public QObject
+class drumkv1_jack_application : public QObject
 {
 	Q_OBJECT
 
 public:
 
 	// Constructor.
-	drumkv1_application(int& argc, char **argv);
+	drumkv1_jack_application(int& argc, char **argv);
 
 	// Destructor.
-	~drumkv1_application();
+	~drumkv1_jack_application();
 
 	// Facade method.
 	int exec();
