@@ -25,6 +25,9 @@
 #include "drumkv1_sched.h"
 #include "drumkv1_param.h"
 
+#include "drumkv1_programs.h"
+#include "drumkv1_controls.h"
+
 #include "lv2/lv2plug.in/ns/ext/midi/midi.h"
 #include "lv2/lv2plug.in/ns/ext/time/time.h"
 #include "lv2/lv2plug.in/ns/ext/atom/util.h"
@@ -98,7 +101,7 @@ private:
 
 drumkv1_lv2::drumkv1_lv2 (
 	double sample_rate, const LV2_Feature *const *host_features )
-	: drumkv1(2, uint32_t(sample_rate))
+	: drumkv1(2, float(sample_rate))
 {
 	m_urid_map = NULL;
 	m_atom_sequence = NULL;
@@ -129,6 +132,9 @@ drumkv1_lv2::drumkv1_lv2 (
 	m_outs = new float * [nchannels];
 	for (uint16_t k = 0; k < nchannels; ++k)
 		m_ins[k] = m_outs[k] = NULL;
+
+	drumkv1::programs()->optional(true);
+	drumkv1::controls()->optional(true);
 }
 
 
@@ -324,7 +330,7 @@ static LV2_State_Status drumkv1_lv2_state_restore ( LV2_Handle instance,
 			drumkv1_param::loadElements(pPlugin, eElements, mapPath);
 	}
 
-	drumkv1_sched::sync_notify(drumkv1_sched::Sample);
+	drumkv1_sched::sync_notify(pPlugin, drumkv1_sched::Sample, 1);
 
 	return LV2_STATE_SUCCESS;
 }
