@@ -42,6 +42,19 @@ public:
 	// dtor.
 	~drumkv1_programs();
 
+	// operational mode flags.
+	enum Mode { Enabled = 1, Optional = 2 };
+
+	void enabled(bool on)
+		{ if (on) m_mode |= Enabled; else m_mode &= ~Enabled; }
+	bool enabled() const
+		{ return m_mode & Enabled; }
+
+	void optional(bool on)
+		{ if (on) m_mode |= Optional; else m_mode &= ~Optional; }
+	bool optional() const
+		{ return m_mode & Optional; }
+
 	// prog. base node
 	class Prog
 	{
@@ -120,8 +133,8 @@ protected:
 	public:
 
 		// ctor.
-		Sched (drumkv1 *pDrumk) : drumkv1_sched(Programs),
-			m_pDrumk(pDrumk), m_bank_id(0), m_prog_id(0) {}
+		Sched (drumkv1 *pDrumk)
+			: drumkv1_sched(pDrumk, Programs), m_bank_id(0), m_prog_id(0) {}
 
 		// schedule (override)
 		void select_program(uint16_t bank_id, uint16_t prog_id)
@@ -133,17 +146,16 @@ protected:
 		}
 
 		// process (virtual).
-		void process()
+		void process(int)
 		{
-			drumkv1_programs *pPrograms = m_pDrumk->programs();
-			pPrograms->process_program(m_pDrumk, m_bank_id, m_prog_id);
+			drumkv1 *pDrumk = instance();
+			drumkv1_programs *pPrograms = pDrumk->programs();
+			pPrograms->process_program(pDrumk, m_bank_id, m_prog_id);
 		}
 
 	private:
 
 		// instance variables.
-		drumkv1 *m_pDrumk;
-
 		uint16_t m_bank_id;
 		uint16_t m_prog_id;
 	};
@@ -151,7 +163,9 @@ protected:
 private:
 
 	// instance variables.
-	Sched *m_sched;
+	unsigned int m_mode;
+
+	Sched m_sched;
 
 	uint8_t m_bank_msb;
 	uint8_t m_bank_lsb;
