@@ -26,6 +26,7 @@
 #include "drumkv1_sched.h"
 
 #include "drumkv1widget_config.h"
+#include "drumkv1widget_control.h"
 
 #include <QMessageBox>
 #include <QDir>
@@ -63,7 +64,7 @@ drumkv1widget::drumkv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 	m_ui.setupUi(this);
 
 	// Init sched notifier.
-	m_sched_notifier = new drumkv1_sched_notifier(this);
+	m_sched_notifier = NULL;
 
 	// Init swapable params A/B to default.
 	for (uint32_t i = 0; i < drumkv1::NUM_PARAMS; ++i)
@@ -476,11 +477,6 @@ drumkv1widget::drumkv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 		SIGNAL(triggered(bool)),
 		SLOT(helpAboutQt()));
 
-	// Special sample update notifications (eg. reverse)
-	QObject::connect(m_sched_notifier,
-		SIGNAL(notify(int)),
-		SLOT(updateSchedNotify(int)));
-
 	// General knob/dial  behavior init...
 	drumkv1_config *pConfig = drumkv1_config::getInstance();
 	if (pConfig) {
@@ -500,7 +496,28 @@ drumkv1widget::drumkv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 // Destructor.
 drumkv1widget::~drumkv1widget (void)
 {
-	delete m_sched_notifier;
+	if (m_sched_notifier)
+		delete m_sched_notifier;
+}
+
+
+// Create/initialize the scheduler/work notifier.
+void drumkv1widget::initSchedNotifier (void)
+{
+	if (m_sched_notifier) {
+		delete m_sched_notifier;
+		m_sched_notifier = NULL;
+	}
+
+	drumkv1_ui *pDrumkUi = ui_instance();
+	if (pDrumkUi == NULL)
+		return;
+
+	m_sched_notifier = new drumkv1widget_sched(pDrumkUi->instance(), this);
+
+	QObject::connect(m_sched_notifier,
+		SIGNAL(notify(int, int)),
+		SLOT(updateSchedNotify(int, int)));
 }
 
 
@@ -515,6 +532,12 @@ void drumkv1widget::setParamKnob ( drumkv1::ParamIndex index, drumkv1widget_knob
 	QObject::connect(pKnob,
 		SIGNAL(valueChanged(float)),
 		SLOT(paramChanged(float)));
+
+	pKnob->setContextMenuPolicy(Qt::CustomContextMenu);
+
+	QObject::connect(pKnob,
+		SIGNAL(customContextMenuRequested(const QPoint&)),
+		SLOT(paramContextMenu(const QPoint&)));
 }
 
 drumkv1widget_knob *drumkv1widget::paramKnob ( drumkv1::ParamIndex index ) const
@@ -563,7 +586,7 @@ void drumkv1widget::paramChanged ( float fValue )
 
 	drumkv1widget_knob *pKnob = qobject_cast<drumkv1widget_knob *> (sender());
 	if (pKnob) {
-		drumkv1::ParamIndex index = m_knobParams.value(pKnob);
+		const drumkv1::ParamIndex index = m_knobParams.value(pKnob);
 		// Save current element param value...
 		drumkv1_ui *pDrumkUi = ui_instance();
 		if (pDrumkUi) {
@@ -575,8 +598,7 @@ void drumkv1widget::paramChanged ( float fValue )
 		// Proceed with regular formalities...
 		updateParam(index, fValue);
 		updateParamEx(index, fValue);
-		m_ui.StatusBar->showMessage(QString("%1 / %2: %3")
-			.arg(m_ui.StackedWidget->currentWidget()->windowTitle())
+		m_ui.StatusBar->showMessage(QString("%1: %2")
 			.arg(pKnob->toolTip())
 			.arg(pKnob->valueText()), 5000);
 		updateDirtyPreset(true);
@@ -590,7 +612,7 @@ void drumkv1widget::updateParamEx ( drumkv1::ParamIndex index, float fValue )
 	++m_iUpdate;
 
 	switch (index) {
-#if 0//--updateSchedNotify(drumkv1_sched::Sample);
+#if 0//--updateSchedNotify(drumkv1_sched::Sample, 0);
 	case drumkv1::GEN1_REVERSE: {
 		drumkv1_ui *pDrumkUi = ui_instance();
 		if (pDrumkUi) {
@@ -613,6 +635,26 @@ void drumkv1widget::updateParamEx ( drumkv1::ParamIndex index, float fValue )
 }
 
 
+// Update scheduled controllers param/knob widgets.
+void drumkv1widget::updateSchedParam ( drumkv1::ParamIndex index, float fValue )
+{
+	++m_iUpdate;
+
+	drumkv1widget_knob *pKnob = paramKnob(index);
+	if (pKnob) {
+		pKnob->setValue(fValue, false);
+		updateParam(index, fValue);
+		updateParamEx(index, fValue);
+		m_ui.StatusBar->showMessage(QString("%1: %2")
+			.arg(pKnob->toolTip())
+			.arg(pKnob->valueText()), 5000);
+		updateDirtyPreset(true);
+	}
+
+	--m_iUpdate;
+}
+
+
 // Reset all param knobs to default values.
 void drumkv1widget::resetParams (void)
 {
@@ -625,7 +667,7 @@ void drumkv1widget::resetParams (void)
 	resetSwapParams();
 
 	for (uint32_t i = 0; i < drumkv1::NUM_PARAMS; ++i) {
-		drumkv1::ParamIndex index = drumkv1::ParamIndex(i);
+		const drumkv1::ParamIndex index = drumkv1::ParamIndex(i);
 		float fValue = drumkv1_param::paramDefaultValue(index);
 		drumkv1widget_knob *pKnob = paramKnob(index);
 		if (pKnob)
@@ -658,7 +700,7 @@ void drumkv1widget::swapParams ( bool bOn )
 		drumkv1_element *element = pDrumkUi->element(key);
 		if (element) {
 			for (uint32_t i = 0; i < drumkv1::NUM_ELEMENT_PARAMS; ++i) {
-				drumkv1::ParamIndex index = drumkv1::ParamIndex(i);
+				const drumkv1::ParamIndex index = drumkv1::ParamIndex(i);
 				drumkv1widget_knob *pKnob = paramKnob(index);
 				if (pKnob) {
 					pKnob->setDefaultValue(element->paramValue(index, 0));
@@ -671,14 +713,14 @@ void drumkv1widget::swapParams ( bool bOn )
 		// Retrieve current element param values...
 		if (element) {
 			for (uint32_t i = 0; i < drumkv1::NUM_ELEMENT_PARAMS; ++i) {
-				drumkv1::ParamIndex index = drumkv1::ParamIndex(i);
+				const drumkv1::ParamIndex index = drumkv1::ParamIndex(i);
 				m_params_ab[index] = element->paramValue(index);
 			}
 		}
 	}
 
 	for (uint32_t i = 0; i < drumkv1::NUM_PARAMS; ++i) {
-		drumkv1::ParamIndex index = drumkv1::ParamIndex(i);
+		const drumkv1::ParamIndex index = drumkv1::ParamIndex(i);
 		drumkv1widget_knob *pKnob = paramKnob(index);
 		if (pKnob) {
 			const float fOldValue = pKnob->value();
@@ -712,7 +754,7 @@ void drumkv1widget::updateParamValues ( uint32_t nparams )
 	drumkv1_ui *pDrumkUi = ui_instance();
 
 	for (uint32_t i = 0; i < nparams; ++i) {
-		drumkv1::ParamIndex index = drumkv1::ParamIndex(i);
+		const drumkv1::ParamIndex index = drumkv1::ParamIndex(i);
 		const float fValue = (pDrumkUi
 			? pDrumkUi->paramValue(index)
 			: drumkv1_param::paramDefaultValue(index));
@@ -730,7 +772,7 @@ void drumkv1widget::resetParamValues ( uint32_t nparams )
 	resetSwapParams();
 
 	for (uint32_t i = 0; i < nparams; ++i) {
-		drumkv1::ParamIndex index = drumkv1::ParamIndex(i);
+		const drumkv1::ParamIndex index = drumkv1::ParamIndex(i);
 		const float fValue = drumkv1_param::paramDefaultValue(index);
 		setParamValue(index, fValue, true);
 		updateParam(index, fValue);
@@ -891,13 +933,13 @@ void drumkv1widget::loadSampleFile ( const QString& sFilename )
 	if (element == NULL) {
 		element = pDrumkUi->addElement(note);
 		for (uint32_t i = 0; i < drumkv1::NUM_ELEMENT_PARAMS; ++i) {
-			drumkv1::ParamIndex index = drumkv1::ParamIndex(i);
+			const drumkv1::ParamIndex index = drumkv1::ParamIndex(i);
 			const float fValue = drumkv1_param::paramDefaultValue(index);
 			element->setParamValue(index, fValue);
 		}
 		pDrumkUi->setCurrentElement(note);
 		for (uint32_t i = 0; i < drumkv1::NUM_ELEMENT_PARAMS; ++i) {
-			drumkv1::ParamIndex index = drumkv1::ParamIndex(i);
+			const drumkv1::ParamIndex index = drumkv1::ParamIndex(i);
 			setParamValue(index, element->paramValue(index));
 		}
 		activateParamKnobs(true);
@@ -913,8 +955,8 @@ void drumkv1widget::loadSampleFile ( const QString& sFilename )
 // Sample updater (crude experimental stuff II).
 void drumkv1widget::updateSample ( drumkv1_sample *pSample, bool bDirty )
 {
-	m_ui.Gen1Sample->setSampleName(currentNoteName());
 	m_ui.Gen1Sample->setSample(pSample);
+	m_ui.Gen1Sample->setSampleName(currentNoteName());
 
 	if (pSample && bDirty)
 		updateDirtyPreset(true);
@@ -1093,7 +1135,7 @@ void drumkv1widget::activateElement ( bool bOpenSample )
 	if (element == NULL && bOpenSample) {
 		element = pDrumkUi->addElement(iCurrentNote);
 		for (uint32_t i = 0; i < drumkv1::NUM_ELEMENT_PARAMS; ++i) {
-			drumkv1::ParamIndex index = drumkv1::ParamIndex(i);
+			const drumkv1::ParamIndex index = drumkv1::ParamIndex(i);
 			const float fValue = drumkv1_param::paramDefaultValue(index);
 			element->setParamValue(index, fValue, 0);
 			element->setParamValue(index, fValue);
@@ -1106,12 +1148,13 @@ void drumkv1widget::activateElement ( bool bOpenSample )
 
 	if (element) {
 		for (uint32_t i = 0; i < drumkv1::NUM_ELEMENT_PARAMS; ++i) {
-			drumkv1::ParamIndex index = drumkv1::ParamIndex(i);
+			const drumkv1::ParamIndex index = drumkv1::ParamIndex(i);
 			drumkv1widget_knob *pKnob = paramKnob(index);
 			if (pKnob)
 				pKnob->setDefaultValue(element->paramValue(index, 0));
 			const float fValue = element->paramValue(index);
 			setParamValue(index, fValue);
+			updateParam(index, fValue);
 		}
 		updateSample(pDrumkUi->sample());
 		refreshElements();
@@ -1161,7 +1204,7 @@ void drumkv1widget::resetElement (void)
 }
 
 
-// (En|Dis)able/ all param/knobs.
+// (En|Dis)able all param/knobs.
 void drumkv1widget::activateParamKnobs ( bool bEnabled )
 {
 	activateParamKnobsGroupBox(m_ui.Gen1GroupBox, bEnabled);
@@ -1174,7 +1217,8 @@ void drumkv1widget::activateParamKnobs ( bool bEnabled )
 }
 
 
-void drumkv1widget::activateParamKnobsGroupBox ( QGroupBox *pGroupBox, bool bEnabled )
+void drumkv1widget::activateParamKnobsGroupBox (
+	QGroupBox *pGroupBox, bool bEnabled )
 {
 	const QList<QWidget *>& children
 		= pGroupBox->findChildren<QWidget *> ();
@@ -1252,17 +1296,31 @@ void drumkv1widget::updateLoadPreset ( const QString& sPreset )
 
 
 // Notification updater.
-void drumkv1widget::updateSchedNotify ( int stype )
+void drumkv1widget::updateSchedNotify ( int stype, int sid )
 {
 	drumkv1_ui *pDrumkUi = ui_instance();
 	if (pDrumkUi == NULL)
 		return;
 
 #ifdef CONFIG_DEBUG
-	qDebug("drumkv1widget::updateSchedNotify(%d)", stype);
+	qDebug("drumkv1widget::updateSchedNotify(%d, %d)", stype, sid);
 #endif
 
 	switch (drumkv1_sched::Type(stype)) {
+	case drumkv1_sched::Controller: {
+		drumkv1widget_control *pInstance
+			= drumkv1widget_control::getInstance();
+		if (pInstance) {
+			drumkv1_controls *pControls = pDrumkUi->controls();
+			pInstance->setControlKey(pControls->current_key());
+		}
+		break;
+	}
+	case drumkv1_sched::Controls: {
+		const drumkv1::ParamIndex index = drumkv1::ParamIndex(sid);
+		updateSchedParam(index, pDrumkUi->paramValue(index));
+		break;
+	}
 	case drumkv1_sched::Programs: {
 		drumkv1_programs *pPrograms = pDrumkUi->programs();
 		drumkv1_programs::Prog *pProg = pPrograms->current_prog();
@@ -1270,7 +1328,14 @@ void drumkv1widget::updateSchedNotify ( int stype )
 		break;
 	}
 	case drumkv1_sched::Sample:
-		updateSample(pDrumkUi->sample());
+		if (sid > 0) {
+		//	refreshElements();
+			activateElement();
+			updateParamValues(drumkv1::NUM_PARAMS);
+			updateDirtyPreset(false);
+		} else {
+			updateSample(pDrumkUi->sample());
+		}
 		// Fall thru...
 	default:
 		break;
@@ -1287,8 +1352,10 @@ void drumkv1widget::helpConfigure (void)
 
 	drumkv1widget_config form(this);
 
-	// Set programs database...
+	// Set controllers&&programs database...
+	form.setControls(pDrumkUi->controls());
 	form.setPrograms(pDrumkUi->programs());
+
 	form.exec();
 }
 
@@ -1357,4 +1424,37 @@ void drumkv1widget::updateDirtyPreset ( bool bDirtyPreset )
 }
 
 
+// Param knob context menu.
+void drumkv1widget::paramContextMenu ( const QPoint& pos )
+{
+	drumkv1widget_knob *pKnob
+		= qobject_cast<drumkv1widget_knob *> (sender());
+	if (pKnob == NULL)
+		return;
+
+	drumkv1_ui *pDrumkUi = ui_instance();
+	if (pDrumkUi == NULL)
+		return;
+
+	drumkv1_controls *pControls = pDrumkUi->controls();
+	if (pControls == NULL)
+		return;
+
+	if (!pControls->enabled())
+		return;
+
+	QMenu menu(this);
+
+	QAction *pAction = menu.addAction(
+		QIcon(":/images/drumkv1_control.png"),
+		tr("MIDI &Controller..."));
+
+	if (menu.exec(pKnob->mapToGlobal(pos)) == pAction) {
+		const drumkv1::ParamIndex index = m_knobParams.value(pKnob);
+		const QString& sTitle = pKnob->toolTip();
+		drumkv1widget_control::showInstance(pControls, index, sTitle, this);
+	}
+}
+
+
 // end of drumkv1widget.cpp
