@@ -1,7 +1,7 @@
 // drumkv1widget_jack.h
 //
 /****************************************************************************
-   Copyright (C) 2012-2014, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2015, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -44,6 +44,9 @@ public:
 	// Constructor.
 	drumkv1widget_jack(drumkv1_jack *pDrumk);
 
+	// Destructor.
+	~drumkv1widget_jack();
+
 #ifdef CONFIG_NSM
 	// NSM client accessors.
 	void setNsmClient(drumkv1_nsm *pNsmClient);
@@ -53,7 +56,7 @@ public:
 protected:
 
 	// Synth engine accessor.
-	drumkv1 *instance() const;
+	drumkv1_ui *ui_instance() const;
 
 	// Param port method.
 	void updateParam(drumkv1::ParamIndex index, float fValue) const;
@@ -73,7 +76,8 @@ protected:
 private:
 
 	// Instance variables.
-	drumkv1_jack *m_pDrumk;
+	drumkv1     *m_pDrumk;
+	drumkv1_ui  *m_pDrumkUi;
 
 #ifdef CONFIG_NSM
 	drumkv1_nsm *m_pNsmClient;
