@@ -48,6 +48,9 @@ drumkv1widget_lv2::drumkv1widget_lv2 ( drumkv1_lv2 *pDrumk,
 	for (uint32_t i = 0; i < drumkv1::NUM_PARAMS; ++i)
 		m_params_def[i] = true;
 
+	// May initialize the scheduler/work notifier.
+	initSchedNotifier();
+
 	// Initial update, always...
 	refreshElements();
 	activateElement();
@@ -119,13 +122,13 @@ void drumkv1widget_lv2::port_event ( uint32_t port_index,
 	uint32_t buffer_size, uint32_t format, const void *buffer )
 {
 	if (format == 0 && buffer_size == sizeof(float)) {
-		drumkv1::ParamIndex index
+		const drumkv1::ParamIndex index
 			= drumkv1::ParamIndex(port_index - drumkv1_lv2::ParamBase);
-		float fValue = *(float *) buffer;
-	//--legacy support < 0.3.0.4 -- begin
+		const float fValue = *(float *) buffer;
+	#if 0//--legacy support < 0.3.0.4 -- begin
 		if (index == drumkv1::DEL1_BPM && fValue < 3.6f)
 			fValue *= 100.0f;
-	//--legacy support < 0.3.0.4 -- end.
+	#endif
 		setParamValue(index, fValue, m_params_def[index]);
 		m_params_def[index] = false;
 	}
