@@ -1,7 +1,7 @@
 // drumkv1widget_lv2.h
 //
 /****************************************************************************
-   Copyright (C) 2012-2014, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2015, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -23,22 +23,7 @@
 #define __drumkv1widget_lv2_h
 
 #include "drumkv1widget.h"
-
-#include "lv2.h"
-#include "lv2/lv2plug.in/ns/extensions/ui/ui.h"
-
-
-#define DRUMKV1_LV2UI_URI DRUMKV1_LV2_PREFIX "ui"
-
-
-// Forward decls.
-class drumkv1_lv2;
-
-
-#ifdef CONFIG_LV2_EXTERNAL_UI
-#include "lv2_external_ui.h"
-#define DRUMKV1_LV2UI_EXTERNAL_URI DRUMKV1_LV2_PREFIX "ui_external"
-#endif
+#include "drumkv1_lv2ui.h"
 
 
 //-------------------------------------------------------------------------
@@ -53,7 +38,10 @@ public:
 	drumkv1widget_lv2(drumkv1_lv2 *pDrumk,
 		LV2UI_Controller controller, LV2UI_Write_Function write_function);
 
-	// Plugin port event notification.
+	// Destructor.
+	~drumkv1widget_lv2();
+
+	// LV2 port event dispatcher.
 	void port_event(uint32_t port_index,
 		uint32_t buffer_size, uint32_t format, const void *buffer);
 
@@ -69,7 +57,7 @@ public:
 protected:
 
 	// Synth engine accessor.
-	drumkv1 *instance() const;
+	drumkv1_ui *ui_instance() const;
 
 	// Param port method.
 	void updateParam(drumkv1::ParamIndex index, float fValue) const;
@@ -82,10 +70,7 @@ protected:
 private:
 
 	// Instance variables.
-	drumkv1_lv2 *m_pDrumk;
-
-	LV2UI_Controller     m_controller;
-	LV2UI_Write_Function m_write_function;
+	drumkv1_lv2ui *m_pDrumkUi;
 
 #ifdef CONFIG_LV2_EXTERNAL_UI
 	LV2_External_UI_Host *m_external_host;
