@@ -1,7 +1,7 @@
 // drumkv1widget_sample.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2014, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2015, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -20,8 +20,8 @@
 *****************************************************************************/
 
 #include "drumkv1widget_sample.h"
-#include "drumkv1widget_config.h"
 
+#include "drumkv1_config.h"
 #include "drumkv1_sample.h"
 
 #include <sndfile.h>
@@ -30,9 +30,13 @@
 
 #include <QApplication>
 #include <QFileDialog>
+#include <QMimeData>
+#include <QDrag>
 #include <QUrl>
 
 #include <QMouseEvent>
+#include <QDragEnterEvent>
+#include <QDropEvent>
 
 #include <QToolTip>
 
@@ -47,7 +51,12 @@ drumkv1widget_sample::drumkv1widget_sample (
 {
 	QFrame::setMouseTracking(true);
 	QFrame::setFocusPolicy(Qt::ClickFocus);
+
 	QFrame::setMinimumSize(QSize(380, 80));
+	QFrame::setSizePolicy(
+		QSizePolicy::Preferred, QSizePolicy::MinimumExpanding);
+
+	QFrame::setAcceptDrops(true);
 
 	QFrame::setFrameShape(QFrame::Panel);
 	QFrame::setFrameShadow(QFrame::Sunken);
@@ -55,6 +64,9 @@ drumkv1widget_sample::drumkv1widget_sample (
 	m_bLoop = false;
 	m_iLoopStart = m_iLoopEnd = 0;
 
+	m_dragCursor  = DragNone;
+	m_pDragSample = NULL;
+
 	resetDragState();
 }
 
@@ -62,7 +74,7 @@ drumkv1widget_sample::drumkv1widget_sample (
 // Destructor.
 drumkv1widget_sample::~drumkv1widget_sample (void)
 {
-	setSample(0);
+	setSample(NULL);
 }
 
 
@@ -80,7 +92,9 @@ void drumkv1widget_sample::setSample ( drumkv1_sample *pSample )
 	m_pSample = pSample;
 
 //	m_bLoop = false;
-	m_iLoopStart = m_iLoopEnd = 0;
+//	m_iLoopStart = m_iLoopEnd = 0;
+
+	m_pDragSample = NULL;
 
 	if (m_pSample)
 		m_iChannels = m_pSample->channels();
@@ -189,13 +203,6 @@ uint32_t drumkv1widget_sample::loopEnd (void) const
 }
 
 
-// Widget resize handler.
-void drumkv1widget_sample::resizeEvent ( QResizeEvent * )
-{
-//	setSample(m_pSample);	-- reset polygon...
-}
-
-
 // Sanitizer helper;
 int drumkv1widget_sample::safeX ( int x ) const
 {
@@ -210,22 +217,27 @@ int drumkv1widget_sample::safeX ( int x ) const
 }
 
 
+// Widget resize handler.
+void drumkv1widget_sample::resizeEvent ( QResizeEvent * )
+{
+	setSample(m_pSample);	// reset polygon...
+}
+
+
 // Mouse interaction.
 void drumkv1widget_sample::mousePressEvent ( QMouseEvent *pMouseEvent )
 {
-	if (m_bLoop) {
-		if (pMouseEvent->button() == Qt::LeftButton) {
-			if (m_dragCursor == DragNone) {
-				m_dragState = DragStart;
-				m_posDrag = pMouseEvent->pos();
-			} else {
-				const int w = QFrame::width();
-				const uint32_t nframes = m_pSample->length();
-				if (nframes > 0) {
-					m_iDragStartX = safeX((m_iLoopStart * w) / nframes);
-					m_iDragEndX   = safeX((m_iLoopEnd   * w) / nframes);
-					m_dragState   = m_dragCursor;
-				}
+	if (pMouseEvent->button() == Qt::LeftButton) {
+		if (m_dragCursor == DragNone) {
+			m_dragState = DragStart;
+			m_posDrag = pMouseEvent->pos();
+		} else if (m_bLoop) {
+			const int w = QFrame::width();
+			const uint32_t nframes = m_pSample->length();
+			if (nframes > 0) {
+				m_iDragStartX = safeX((m_iLoopStart * w) / nframes);
+				m_iDragEndX   = safeX((m_iLoopEnd   * w) / nframes);
+				m_dragState   = m_dragCursor;
 			}
 		}
 	}
@@ -322,9 +334,22 @@ void drumkv1widget_sample::mouseMoveEvent ( QMouseEvent *pMouseEvent )
 		if ((m_posDrag - pMouseEvent->pos()).manhattanLength()
 			> QApplication::startDragDistance()) {
 			// Start dragging alright...
-			m_dragState = m_dragCursor = DragSelect;
-			m_iDragStartX = m_iDragEndX = m_posDrag.x();
-			QFrame::setCursor(QCursor(Qt::SizeHorCursor));
+			if (m_bLoop && (pMouseEvent->modifiers()
+				& (Qt::ShiftModifier | Qt::ControlModifier))) {
+				m_dragState = m_dragCursor = DragSelect;
+				m_iDragStartX = m_iDragEndX = m_posDrag.x();
+				QFrame::setCursor(QCursor(Qt::SizeHorCursor));
+			} else if (m_pSample && m_pSample->filename()) {
+				QList<QUrl> urls;
+				m_pDragSample = m_pSample;
+				urls.append(QUrl::fromLocalFile(m_pDragSample->filename()));
+				QMimeData *pMimeData = new QMimeData();
+				pMimeData->setUrls(urls);;
+				QDrag *pDrag = new QDrag(this);
+				pDrag->setMimeData(pMimeData);
+				pDrag->exec(Qt::CopyAction);
+				resetDragState();
+			}
 		}
 		// Fall thru...
 	default:
@@ -378,7 +403,7 @@ void drumkv1widget_sample::mouseReleaseEvent ( QMouseEvent *pMouseEvent )
 		break;
 	}
 
-
+	m_pDragSample = NULL;
 	resetDragState();
 }
 
@@ -394,6 +419,7 @@ void drumkv1widget_sample::keyPressEvent ( QKeyEvent *pKeyEvent )
 {
 	switch (pKeyEvent->key()) {
 	case Qt::Key_Escape:
+		m_pDragSample = NULL;
 		resetDragState();
 		update();
 		break;
@@ -404,6 +430,33 @@ void drumkv1widget_sample::keyPressEvent ( QKeyEvent *pKeyEvent )
 }
 
 
+// Drag-n-drop (more of the later) support.
+void drumkv1widget_sample::dragEnterEvent ( QDragEnterEvent *pDragEnterEvent )
+{
+	QFrame::dragEnterEvent(pDragEnterEvent);
+
+	if (m_pDragSample && m_pDragSample == sample())
+		return;
+
+	if (pDragEnterEvent->mimeData()->hasUrls())
+		pDragEnterEvent->acceptProposedAction();
+}
+
+
+void drumkv1widget_sample::dropEvent ( QDropEvent *pDropEvent )
+{
+	QFrame::dropEvent(pDropEvent);
+
+	const QMimeData *pMimeData = pDropEvent->mimeData();
+	if (pMimeData->hasUrls()) {
+		const QString& sFilename
+			= QListIterator<QUrl>(pMimeData->urls()).peekNext().toLocalFile();
+		if (!sFilename.isEmpty())
+			emit loadSampleFile(sFilename);
+	}
+}
+
+
 // Reset drag/select state.
 void drumkv1widget_sample::resetDragState (void)
 {
@@ -473,7 +526,7 @@ void drumkv1widget_sample::paintEvent ( QPaintEvent *pPaintEvent )
 	} else {
 		painter.setPen(pal.midlight().color());
 		painter.drawText(rect, Qt::AlignCenter,
-			tr("(double-click to load new sample...)"));
+			tr("(double-click or drop to load new sample...)"));
 	}
 
 	painter.end();
@@ -485,7 +538,7 @@ void drumkv1widget_sample::paintEvent ( QPaintEvent *pPaintEvent )
 // Browse for a new sample.
 void drumkv1widget_sample::openSample ( const QString& sName )
 {
-	drumkv1widget_config *pConfig = drumkv1widget_config::getInstance();
+	drumkv1_config *pConfig = drumkv1_config::getInstance();
 	if (pConfig == NULL)
 		return;
 
@@ -585,4 +638,11 @@ void drumkv1widget_sample::updateToolTip (void)
 }
 
 
+// Default size hint.
+QSize drumkv1widget_sample::sizeHint (void) const
+{
+	return QSize(380, 80);
+}
+
+
 // end of drumkv1widget_sample.cpp
