@@ -1,7 +1,7 @@
 // drumkv1widget_sample.h
 //
 /****************************************************************************
-   Copyright (C) 2012-2014, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2015, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -30,6 +30,9 @@
 // Forward decl.
 class drumkv1_sample;
 
+class QDragEnterEvent;
+class QDropEvent;
+
 
 //----------------------------------------------------------------------------
 // drumkv1widget_sample -- Custom widget
@@ -81,6 +84,9 @@ public slots:
 
 protected:
 
+	// Sanitizer helper;
+	int safeX(int x) const;
+
 	// Widget resize handler.
 	void resizeEvent(QResizeEvent *);
 
@@ -97,8 +103,9 @@ protected:
 	// Trap for escape key.
 	void keyPressEvent(QKeyEvent *pKeyEvent);
 
-	// Sanitizer helper;
-	int safeX(int x) const;
+	// Drag-n-drop (more of the later) support.
+	void dragEnterEvent(QDragEnterEvent *pDragEnterEvent);
+	void dropEvent(QDropEvent *pDropEvent);
 
 	// Reset drag/select state.
 	void resetDragState();
@@ -106,6 +113,9 @@ protected:
 	// Update tool-tip.
 	void updateToolTip();
 
+	// Default size hint.
+	QSize sizeHint() const;
+
 private:
 
 	// Instance state.
@@ -126,6 +136,8 @@ private:
 	int m_iDragStartX;
 	int m_iDragEndX;
 
+	drumkv1_sample *m_pDragSample;
+
 	// Loop state.
 	bool     m_bLoop;
 	uint32_t m_iLoopStart;
